/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   Linux, i386 - specific code
 *   this file is a part of xtrace project
 *
 * $Id: linux-unknown.c,v 1.2 2002/05/02 14:02:18 drozd Exp $
 */


#include <sys/ptrace.h>
#include <linux/user.h>

#include <stdio.h>
#include <string.h>

#include "xtrace.h"
#include "linux.h"


static struct user_regs_struct *getregs(const struct opt *);
static long read_dword(const struct opt *, vaddr_t);

/*
 * Get general purpose registers
 */
struct user_regs_struct *
getregs(const struct opt *op)
{
	static struct user_regs_struct regs;
	
	/* copy the registers */
	if (ptrace(PTRACE_GETREGS, op->pid, 0, &regs) != 0) {
		perror("PTRACE_GETREGS"); return 0;
	}

	return &regs;

}

/*
 * Read one double word from text segment
 */
long
read_dword(const struct opt *op, vaddr_t loc)
{
	long ret = 0;

	ret = ptrace(PTRACE_PEEKTEXT, op->pid, loc, 0);

	return ret;
}

/*
 * Get call data by stack pointer
 * this function must set the following fields:
 *   o  op->scno
 *   o  all fields from calldata structure
 */
int
xtr_get_call_data(struct opt *op, struct calldata *cdp)
{
	struct user_regs_struct *rp;
	static vaddr_t *argv[MAXARGS], args[MAXARGS];
	int i;

	if (cdp == 0)
		return (-1);

	args[0] = 0;

	if (argv[0] == NULL)
		for (i = 0; i < sizeof(argv)/sizeof(argv[0]); i++)
			argv[i] = &args[i];

	if ((rp = getregs(op)) == 0)
		return (-1);

	/* fill up call data */
	cdp->ip = rp->eip;

	if (cdp->fp == 0)
		cdp->fp = rp->ebp;

	cdp->ofp = read_dword(op, cdp->fp);
	cdp->rta = read_dword(op, cdp->fp + 4);
	cdp->argv = argv;
	cdp->retval = rp->eax;

	op->scno = rp->orig_eax;

	/* handle fuction entrance */
	if (op->flags & XTR_INFUNC) {
		/* FIXME */

	} else {
		if (rp->eax < 0) {
			/* syscall entrance */
			op->flags &= ~XTR_INSYSEXIT;
			op->flags |= XTR_INSYSENTER;
		} else {
			op->flags |= XTR_INSYSEXIT;
			op->flags &= ~XTR_INSYSENTER;
		}

		/* parameters are in registers */
		args[0] = rp->ebx;
		args[1] = rp->ecx;
		args[2] = rp->edx;
		args[3] = rp->esi;
		args[4] = rp->edi;
		args[5] = rp->ebp;
		args[6] = 0;
	}

	return (0);
}
