/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   shared library - related routines
 *   this file is a part of xtrace project
 *
 * $Id: sharedlibs.c,v 1.1 2002/05/07 14:04:21 drozd Exp $
 */

#include <stdio.h>
#include <string.h>

#include "xtrace.h"

/*
 * get list of dependant ELF binaries
 */
struct sharedlib **
xtr_get_deps(pid_t pid)
{
	static struct sharedlib libs[128], *plibs[128];
	char buf[1024], fname[1024], progname[1024];
	FILE *fp;
	int nbytes, i;
	vaddr_t base;
	
	/* read the executable full path */
#ifdef FREEBSD
	(void) snprintf(fname, sizeof(fname), "/proc/%d/file", pid);
#elif LINUX
	(void) snprintf(fname, sizeof(fname), "/proc/%d/exe", pid);
#endif
	nbytes = readlink(fname, progname, sizeof(progname));
	progname[nbytes] = '\0';

	libs[0].path = strdup(progname);
	libs[0].base = 0U;

	/* set tracing flags for dynamic linker */
	(void) setenv("LD_TRACE_LOADED_OBJECTS", "1", 1);
	(void) setenv("LD_TRACE_LOADED_OBJECTS_FMT1", "%p %x\n", 1);
	(void) unsetenv("LD_TRACE_LOADED_OBJECTS_FMT2");

	if ((fp = popen(progname, "r")) == NULL)
		return (NULL);

	for (i = 1; fgets(buf, sizeof(buf), fp) != NULL; ) {
		if (sscanf(buf, "%s %x", fname, &base) == 2) {
			libs[i].path = strdup(fname);
			libs[i].base = base;
			i++;
		}
	}
	
	/* null-terminate the array */
	plibs[i--] = NULL;
	
	for (; i >= 0; i--)
		plibs[i] = &libs[i];

	/* restore environment */
	(void) unsetenv("LD_TRACE_LOADED_OBJECTS");

	return (plibs);
}
